unit CMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants,
{$IFDEF Android}
  Androidapi.JNI.Net, Androidapi.Helpers,
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.Provider, Androidapi.JNIBridge,
  Androidapi.JNI.JavaTypes,
{$ELSE}
  FMX.Platform.iOS,
  FMX.Helpers.iOS,
  Macapi.Helpers,
{$ENDIF}
  FMX.Platform,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls;

type
  TFormMain = class(TForm)
    tmrUpdate: TTimer;
    Button1: TButton;
    procedure tmrUpdateTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function HandleAppEvent(AAppEvent: TApplicationEvent;
      AContext: TObject): Boolean;
  end;

type
  JEnvironment = interface;

  JEnvironmentClass = interface(JObjectClass)
    ['{D131F4D4-A6AD-43B7-B2B6-A9222BC46C74}']
    function _GetMEDIA_BAD_REMOVAL: JString; cdecl;
    function _GetMEDIA_CHECKING: JString; cdecl;
    function _GetMEDIA_EJECTING: JString; cdecl;
    function _GetMEDIA_MOUNTED: JString; cdecl;
    function _GetMEDIA_MOUNTED_READ_ONLY: JString; cdecl;
    function _GetMEDIA_NOFS: JString; cdecl;
    function _GetMEDIA_REMOVED: JString; cdecl;
    function _GetMEDIA_SHARED: JString; cdecl;
    function _GetMEDIA_UNKNOWN: JString; cdecl;
    function _GetMEDIA_UNMOUNTABLE: JString; cdecl;
    function _GetMEDIA_UNMOUNTED: JString; cdecl;
    function _GetDIRECTORY_ALARMS: JString; cdecl;
    function _GetDIRECTORY_DCIM: JString; cdecl;
    function _GetDIRECTORY_DOCUMENTS: JString; cdecl;
    function _GetDIRECTORY_DOWNLOADS: JString;
    function _GetDIRECTORY_MOVIES: JString; cdecl;
    function _GetDIRECTORY_MUSIC: JString; cdecl;
    function _GetDIRECTORY_NOTIFICATIONS: JString; cdecl;
    function _GetDIRECTORY_PICTURES: JString; cdecl;
    function _GetDIRECTORY_PODCASTS: JString; cdecl;
    function _GetDIRECTORY_RINGTONES: JString; cdecl;
    { class } function init: JEnvironment; cdecl;
    { class } function getDataDirectory: JFile; cdecl;
    { class } function getDownloadCacheDirectory: JFile; cdecl;
    { class } function getExternalStorageDirectory(): JFile; cdecl;
    { class } function getExternalStoragePublicDirectory(_type: JString)
      : JFile; cdecl;
    { class } function getExternalStorageState(path: JFile): JString;
      cdecl; overload;
    { class } function getExternalStorageState: JString; cdecl; overload;
    { class } function getRootDirectory: JFile; cdecl;
    { class } function getStorageState(path: JFile): JString; cdecl;
    { class } function isExternalStorageEmulated: Boolean; cdecl; overload;
    { class } function isExternalStorageEmulated(path: JFile): Boolean;
      cdecl; overload;
    { class } function isExternalStorageRemovable(path: JFile): Boolean;
      cdecl; overload;
    { class } function isExternalStorageRemovable: Boolean; cdecl; overload;
    { class } property MEDIA_BAD_REMOVAL: JString read _GetMEDIA_BAD_REMOVAL;
    { class } property MEDIA_CHECKING: JString read _GetMEDIA_CHECKING;
    { class } property MEDIA_EJECTING: JString read _GetMEDIA_EJECTING;
    { class } property MEDIA_MOUNTED: JString read _GetMEDIA_MOUNTED;
    { class } property MEDIA_MOUNTED_READ_ONLY: JString
      read _GetMEDIA_MOUNTED_READ_ONLY;
    { class } property MEDIA_NOFS: JString read _GetMEDIA_NOFS;
    { class } property MEDIA_REMOVED: JString read _GetMEDIA_REMOVED;
    { class } property MEDIA_SHARED: JString read _GetMEDIA_SHARED;
    { class } property MEDIA_UNKNOWN: JString read _GetMEDIA_UNKNOWN;
    { class } property MEDIA_UNMOUNTABLE: JString read _GetMEDIA_UNMOUNTABLE;
    { class } property MEDIA_UNMOUNTED: JString read _GetMEDIA_UNMOUNTED;
    { class } property DIRECTORY_ALARMS: JString read _GetDIRECTORY_ALARMS;
    { class } property DIRECTORY_DCIM: JString read _GetDIRECTORY_DCIM;
    { class } property DIRECTORY_DOCUMENTS: JString
      read _GetDIRECTORY_DOCUMENTS;
    { class } property DIRECTORY_DOWNLOADS: JString
      read _GetDIRECTORY_DOWNLOADS;
    { class } property DIRECTORY_MOVIES: JString read _GetDIRECTORY_MOVIES;
    { class } property DIRECTORY_MUSIC: JString read _GetDIRECTORY_MUSIC;
    { class } property DIRECTORY_NOTIFICATIONS: JString
      read _GetDIRECTORY_NOTIFICATIONS;
    { class } property DIRECTORY_PICTURES: JString read _GetDIRECTORY_PICTURES;
    { class } property DIRECTORY_PODCASTS: JString read _GetDIRECTORY_PODCASTS;
    { class } property DIRECTORY_RINGTONES: JString
      read _GetDIRECTORY_RINGTONES;
  end;

  [JavaSignature('android/os/Environment')]
  JEnvironment = interface(JObject)
    ['{83A2E94E-7D8E-432F-BE21-AEC2115015BE}']
  end;

  TJEnvironment = class(TJavaGenericImport<JEnvironmentClass, JEnvironment>);

var
  FormMain: TFormMain;

implementation

{$R *.fmx}

procedure TFormMain.FormCreate(Sender: TObject);
var
  aFMXApplicationEventService: IFMXApplicationEventService;
begin
  if TPlatformServices.Current.SupportsPlatformService
    (IFMXApplicationEventService, IInterface(aFMXApplicationEventService)) then
    aFMXApplicationEventService.SetApplicationEventHandler(HandleAppEvent);
end;

function TFormMain.HandleAppEvent(AAppEvent: TApplicationEvent;
  AContext: TObject): Boolean;
{$IFDEF IOS}
var
  urlContext: TiOSOpenApplicationContext;
begin
  case AAppEvent of
    TApplicationEvent.OpenURL:
      begin
        urlContext := TiOSOpenApplicationContext(AContext);
        TThread.Queue(nil,
          procedure
          begin
            ShowMessage(urlContext.URL);
          end);
      end;
  end;
  Result := True;
end;
{$ELSE}

begin

end;
{$ENDIF}

function getDataColumn(uri: Jnet_Uri; selection: string;
selectionArgs: string): string;
var
  cursor: JCursor;
  column: string;
  query:JString;
  args,projection: TJavaObjectArray<JString>;
  column_index:integer;
begin
  column := '_data';
  projection := TJavaObjectArray<JString>.Create(1);
  projection[0]:=StringToJString(column);
  args:=nil;
  query:=nil;
  if selection<>'' then
  begin
    query:=StringToJString(selection);
  end;
  if selectionArgs<>'' then
  begin
    args:=TJavaObjectArray<JString>.Create(1);
    args[0]:=StringToJString(selectionArgs);
  end;
  try
    cursor := SharedActivity.getContentResolver().query(uri, projection, query,
      args, nil);
    if (cursor <>nil) and cursor.moveToFirst() then
    begin
      column_index := cursor.getColumnIndexOrThrow(StringToJString(column));
      Exit(JStringToString(cursor.getString(column_index)));
    end;
  finally
    if (cursor <> nil) then
      cursor.close();
  end;
  Exit('');
end;

function isExternalStorageDocument(uri: Jnet_Uri): Boolean;
begin
  Result := JStringToString(uri.getAuthority)
    = 'com.android.externalstorage.documents';
end;

function isDownloadsDocument(uri: Jnet_Uri): Boolean;
begin
  Result := JStringToString(uri.getAuthority)
    = 'com.android.providers.downloads.documents';
end;

function isMediaDocument(uri: Jnet_Uri): Boolean;
begin
  Result := JStringToString(uri.getAuthority)
    = 'com.android.providers.media.documents';
end;

function GetRealPath(uri: Jnet_Uri): string;
var
  docid: string;
  fileType: string;
  split: TArray<string>;
  contentUri: Jnet_Uri;
  selection:string;
  selectionArgs:string;
begin
  /// / DocumentProvider
  if TJDocumentsContract.JavaClass.isDocumentUri(SharedActivityContext, uri)
  then
  begin
    // ExternalStorageProvider
    if (isExternalStorageDocument(uri)) then
    begin
      docid := JStringToString
        (TJDocumentsContract.JavaClass.getDocumentId(uri));
      split := docid.split([':']);
      fileType := split[0];

      if ('primary' = fileType) then
        Exit(JStringToString(TJEnvironment.JavaClass.getExternalStorageDirectory
          ().toString) + '/' + split[1]);
      // TODO handle non-primary volumes
    end
    // DownloadsProvider
    else if (isDownloadsDocument(uri)) then
    begin
      docid := JStringToString
        (TJDocumentsContract.JavaClass.getDocumentId(uri));
      contentUri := TJContentUris.JavaClass.withAppendedId
        (TJnet_Uri.JavaClass.parse
        (StringToJString('content://downloads/public_downloads')),
        docid.ToInt64());

      Exit(getDataColumn(contentUri, '', ''));
    end
    // MediaProvider
    else if (isMediaDocument(uri)) then
    begin
      docId := JStringToString
        (TJDocumentsContract.JavaClass.getDocumentId(uri));
      split := docid.split([':']);
      fileType := split[0];

      contentUri := nil;
      if ('image'=filetype) then
        contentUri := TJImages_Media.Javaclass.EXTERNAL_CONTENT_URI
      else if ('video'=filetype) then
        contentUri := TJVideo_Media.JavaClass.EXTERNAL_CONTENT_URI
      else if ('audio'=filetype) then
        contentUri := TJAudio_Media.JavaClass.EXTERNAL_CONTENT_URI;

      selection := '_id=?';
      selectionArgs := split[1];

      Exit(getDataColumn(contentUri, selection, selectionArgs));
    end;
  end
  // MediaStore (and general)
  else if ('content'=JStringToString(uri.getScheme()).ToLower) then
  begin
    Exit(getDataColumn(uri, '', ''));
  end
  // File
  else if ('file'=JStringToString(uri.getScheme()).ToLower) then
    Exit(JStringToString(uri.getPath()));
end;

procedure TFormMain.tmrUpdateTimer(Sender: TObject);
{$IFDEF Android}
var
  data: JIntent;
  uri: Jnet_Uri;
begin
  //
  data := SharedActivity.getIntent;
  if assigned(data) and assigned(data.getAction()) and
    data.getAction.equals(TJIntent.JavaClass.ACTION_VIEW) then
  begin
    //
    uri := data.getData;
    if assigned(uri) then
    begin
      // TThread.Queue(nil, procedure
      // begin
      ShowMessage(JStringToString(uri.getPath));
      data.setData(nil);
      // end);
    end;
  end;
end;
{$ELSE}

begin
  //

end;
{$ENDIF}

end.
